from urllib.parse import urlencode

from django.urls import reverse
from django.core import signing

from rest_framework.fields import FileField

from yediemin.storages import PrivateS3Boto3Storage


class YedieminFieldMixin:

    def __init__(self, *args, **kwargs):
        self.storage = PrivateS3Boto3Storage
        super().__init__(*args, **kwargs)

    def to_representation(self, value):
        file_url = super().to_representation(value)

        request = self.context["request"]

        key = signing.dumps({
            # Passing to url path.
            "file_name": value.name,

            # Redirection
            "file_url": file_url,

            # Authentication
            "user_id": request.user.id
        })

        yediemin_base_url = request.build_absolute_uri(reverse("yediemin", kwargs={"file_name": value.name}))
        query_string = urlencode({"key": key})
        url = f"{yediemin_base_url}?{query_string}"
        return url


class YedieminFileField(YedieminFieldMixin, FileField):
    pass



from storages.backends.s3boto3 import S3Boto3Storage

from yediemin.settings import EXPIRE_IN


class PrivateS3Boto3Storage(S3Boto3Storage):
    default_acl = "private"
    querystring_auth = True
    querystring_expire = EXPIRE_IN



from django.conf import settings
from rest_framework.authentication import SessionAuthentication

HIDDEN_REDIRECT_PATH = getattr(settings, "YEDIEMIN_HIDDEN_REDIRECT_PATH", "yediemin-files")
AUTHENTICATION_CLASS = getattr(settings, "YEDIEMIN_AUTHENTICATION_CLASS", SessionAuthentication)
EXPIRE_IN = getattr(settings, "YEDIEMIN_EXPIRE_IN", 2592000)
